from _typeshed import Incomplete
from typing import Optional

class TerseExecutableError(Exception):
    value: Incomplete
    def __init__(self, value: str) -> None: ...

class TerseExecutableParser:
    def __init__(self, image_path: Optional[str] = ..., blob: Optional[bytes] = ...) -> None: ...
    @property
    def signature(self) -> str: ...
    @property
    def machine(self) -> int: ...
    @property
    def number_of_sections(self) -> int: ...
    @property
    def subsystem(self) -> int: ...
    @property
    def stripped_size(self) -> int: ...
    @property
    def address_of_entry_point(self) -> int: ...
    @property
    def base_of_code(self) -> int: ...
    @property
    def image_base(self) -> int: ...
